@@ -75,7 +75,7 @@ typedef struct {
 /*
  * prefix that distinguishes mod_auth_openidc cache files from other files in the same directory (/tmp)
  */
-#define OIDC_CACHE_FILE_PREFIX "mod-auth-connect-"
+#define OIDC_CACHE_FILE_PREFIX "mod-auth-openidc-"
 
 /* post config routine */
 int oidc_cache_file_post_config(server_rec *s) {
@@ -95,7 +95,7 @@ int oidc_cache_file_post_config(server_rec *s) {
 static const char *oidc_cache_file_name(request_rec *r, const char *section,
 		const char *key) {
 	return apr_psprintf(r->pool, "%s%s-%s", OIDC_CACHE_FILE_PREFIX, section,
-			key);
+			oidc_util_escape_string(r, key));
 }
 
 /*
@@ -184,7 +184,7 @@ static apr_byte_t oidc_cache_file_get(request_rec *r, const char *section,
 
 	/* open the cache file if it exists, otherwise we just have a "regular" cache miss */
 	if (apr_file_open(&fd, path, APR_FOPEN_READ | APR_FOPEN_BUFFERED,
-	APR_OS_DEFAULT, r->pool) != APR_SUCCESS) {
+			APR_OS_DEFAULT, r->pool) != APR_SUCCESS) {
 		oidc_debug(r, "cache miss for key \"%s\"", key);
 		return TRUE;
 	}
@@ -239,11 +239,6 @@ static apr_byte_t oidc_cache_file_get(request_rec *r, const char *section,
 	apr_file_unlock(fd);
 	apr_file_close(fd);
 
-	/* log a successful cache hit */
-	oidc_debug(r,
-			"cache hit for key \"%s\" (%" APR_SIZE_T_FMT " bytes, expiring in: %" APR_TIME_T_FMT ")",
-			key, info.len, apr_time_sec(info.expire - apr_time_now()));
-
 	return TRUE;
 
 error_close:
@@ -295,6 +290,8 @@ static apr_status_t oidc_cache_file_clean(request_rec *r) {
 		/* time to clean, reset the modification time of the metadata file to reflect the timestamp of this cleaning cycle */
 		apr_file_mtime_set(metadata_path, apr_time_now(), r->pool);
 
+		oidc_debug(r, "start cleaning cycle");
+
 	} else {
 
 		/* no metadata file exists yet, create one (and open it) */
@@ -329,7 +326,7 @@ static apr_status_t oidc_cache_file_clean(request_rec *r) {
 		if (i == APR_SUCCESS) {
 
 			/* skip non-cache entries, cq. the ".", ".." and the metadata file */
-			if ((fi.name[0] == '.')
+			if ((fi.name[0] == OIDC_CHAR_DOT)
 					|| (strstr(fi.name, OIDC_CACHE_FILE_PREFIX) != fi.name)
 					|| ((apr_strnatcmp(fi.name,
 							oidc_cache_file_name(r, "cache-file",
@@ -412,7 +409,8 @@ static apr_byte_t oidc_cache_file_set(request_rec *r, const char *section,
 	}
 
 	/* try to open the cache file for writing, creating it if it does not exist */
-	if ((rc = apr_file_open(&fd, path, (APR_FOPEN_WRITE | APR_FOPEN_CREATE | APR_FOPEN_TRUNCATE),
+	if ((rc = apr_file_open(&fd, path,
+			(APR_FOPEN_WRITE | APR_FOPEN_CREATE | APR_FOPEN_TRUNCATE),
 			APR_OS_DEFAULT, r->pool)) != APR_SUCCESS) {
 		oidc_error(r, "cache file \"%s\" could not be opened (%s)", path,
 				apr_strerror(rc, s_err, sizeof(s_err)));
@@ -435,25 +433,24 @@ static apr_byte_t oidc_cache_file_set(request_rec *r, const char *section,
 		return FALSE;
 
 	/* next write the value */
-	if ((rc = oidc_cache_file_write(r, path, fd, (void *) value, info.len))
-			!= APR_SUCCESS)
-		return FALSE;
+	rc = oidc_cache_file_write(r, path, fd, (void *) value, info.len);
 
 	/* unlock and close the written file */
 	apr_file_unlock(fd);
 	apr_file_close(fd);
 
-	/* log our success */
+	/* log our success/failure */
 	oidc_debug(r,
-			"set entry for key \"%s\" (%" APR_SIZE_T_FMT " bytes, expires in: %" APR_TIME_T_FMT ")",
-			key, info.len, apr_time_sec(expiry - apr_time_now()));
+			"%s entry for key \"%s\" in file of %" APR_SIZE_T_FMT " bytes",
+			(rc == APR_SUCCESS) ? "successfully stored" : "could not store",
+					key, info.len);
 
-	return TRUE;
+	return (rc == APR_SUCCESS);
 }
 
 oidc_cache_t oidc_cache_file = {
+		"file",
 		1,
-		NULL,
 		oidc_cache_file_post_config,
 		NULL,
 		oidc_cache_file_get,
