@@ -97,16 +97,16 @@ static int oidc_cache_memcache_post_config(server_rec *s) {
 
 	if (cfg->cache_memcache_servers == NULL) {
 		oidc_serror(s,
-				"cache type is set to \"memcache\", but no valid OIDCMemCacheServers setting was found");
+				"cache type is set to \"memcache\", but no valid " OIDCMemCacheServers " setting was found");
 		return HTTP_INTERNAL_SERVER_ERROR;
 	}
 
 	/* loop over the provided memcache servers to find out the number of servers configured */
 	char *cache_config = apr_pstrdup(p, cfg->cache_memcache_servers);
-	split = apr_strtok(cache_config, " ", &tok);
+	split = apr_strtok(cache_config, OIDC_STR_SPACE, &tok);
 	while (split) {
 		nservers++;
-		split = apr_strtok(NULL, " ", &tok);
+		split = apr_strtok(NULL, OIDC_STR_SPACE, &tok);
 	}
 
 	/* allocated space for the number of servers */
@@ -119,7 +119,7 @@ static int oidc_cache_memcache_post_config(server_rec *s) {
 
 	/* loop again over the provided servers */
 	cache_config = apr_pstrdup(p, cfg->cache_memcache_servers);
-	split = apr_strtok(cache_config, " ", &tok);
+	split = apr_strtok(cache_config, OIDC_STR_SPACE, &tok);
 	while (split) {
 		apr_memcache_server_t* st;
 		char* host_str;
@@ -160,7 +160,7 @@ static int oidc_cache_memcache_post_config(server_rec *s) {
 		}
 
 		/* go to the next entry */
-		split = apr_strtok(NULL, " ", &tok);
+		split = apr_strtok(NULL, OIDC_STR_SPACE, &tok);
 	}
 
 	return OK;
@@ -188,14 +188,26 @@ static char *oidc_cache_memcache_get_key(apr_pool_t *pool, const char *section,
 	return apr_psprintf(pool, "%s:%s", section, key);
 }
 
+/*
+ * check dead/alive status for all servers
+ */
+static apr_byte_t oidc_cache_memcache_status(request_rec *r,
+		oidc_cache_cfg_memcache_t *context) {
+	int rc = TRUE;
+	int i;
+	for (i = 0; rc && i < context->cache_memcache->ntotal; i++)
+		rc = rc
+		&& (context->cache_memcache->live_servers[0]->status
+				!= APR_MC_SERVER_DEAD);
+	return rc;
+}
+
 /*
  * get a name/value pair from memcache
  */
 static apr_byte_t oidc_cache_memcache_get(request_rec *r, const char *section,
 		const char *key, const char **value) {
 
-	oidc_debug(r, "enter, section=\"%s\", key=\"%s\"", section, key);
-
 	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
 			&auth_openidc_module);
 	oidc_cache_cfg_memcache_t *context =
@@ -209,11 +221,26 @@ static apr_byte_t oidc_cache_memcache_get(request_rec *r, const char *section,
 			&len, NULL);
 
 	if (rv == APR_NOTFOUND) {
+
+		/*
+		 * NB: workaround the fact that the apr_memcache returns APR_NOTFOUND if a server has been marked dead
+		 */
+		if (oidc_cache_memcache_status(r, context) == FALSE) {
+
+			oidc_cache_memcache_log_status_error(r, "apr_memcache_getp", rv);
+
+			return FALSE;
+		}
+
 		oidc_debug(r, "apr_memcache_getp: key %s not found in cache",
 				oidc_cache_memcache_get_key(r->pool, section, key));
-		return FALSE;
+
+		return TRUE;
+
 	} else if (rv != APR_SUCCESS) {
+
 		oidc_cache_memcache_log_status_error(r, "apr_memcache_getp", rv);
+
 		return FALSE;
 	}
 
@@ -234,8 +261,6 @@ static apr_byte_t oidc_cache_memcache_get(request_rec *r, const char *section,
 static apr_byte_t oidc_cache_memcache_set(request_rec *r, const char *section,
 		const char *key, const char *value, apr_time_t expiry) {
 
-	oidc_debug(r, "enter, section=\"%s\", key=\"%s\"", section, key);
-
 	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
 			&auth_openidc_module);
 	oidc_cache_cfg_memcache_t *context =
@@ -252,6 +277,7 @@ static apr_byte_t oidc_cache_memcache_set(request_rec *r, const char *section,
 		if (rv == APR_NOTFOUND) {
 			oidc_debug(r, "apr_memcache_delete: key %s not found in cache",
 					oidc_cache_memcache_get_key(r->pool, section, key));
+			rv = APR_SUCCESS;
 		} else if (rv != APR_SUCCESS) {
 			oidc_cache_memcache_log_status_error(r, "apr_memcache_delete", rv);
 		}
@@ -275,8 +301,8 @@ static apr_byte_t oidc_cache_memcache_set(request_rec *r, const char *section,
 }
 
 oidc_cache_t oidc_cache_memcache = {
+		"memcache",
 		1,
-		oidc_cache_memcache_cfg_create,
 		oidc_cache_memcache_post_config,
 		NULL,
 		oidc_cache_memcache_get,
