@@ -100,7 +100,7 @@ static int oidc_cache_redis_post_config(server_rec *s) {
 	/* parse the host:post tuple from the configuration */
 	if (cfg->cache_redis_server == NULL) {
 		oidc_serror(s,
-				"cache type is set to \"redis\", but no valid OIDCRedisCacheServer setting was found");
+				"cache type is set to \"redis\", but no valid " OIDCRedisCacheServer " setting was found");
 		return HTTP_INTERNAL_SERVER_ERROR;
 	}
 
@@ -124,7 +124,8 @@ static int oidc_cache_redis_post_config(server_rec *s) {
 		context->port = 6379;
 
 	if (cfg->cache_redis_password != NULL) {
-		context->passwd = apr_pstrdup(s->process->pool, cfg->cache_redis_password);
+		context->passwd = apr_pstrdup(s->process->pool,
+				cfg->cache_redis_password);
 	}
 
 	if (oidc_cache_mutex_post_config(s, context->mutex, "redis") == FALSE)
@@ -156,39 +157,66 @@ static char *oidc_cache_redis_get_key(apr_pool_t *pool, const char *section,
 /* key for storing data in the process pool */
 #define OIDC_CACHE_REDIS_CONTEXT "oidc_cache_redis_context"
 
+/*
+ * per-process Redis connection context
+ */
+typedef struct {
+	redisContext *ctx;
+} oidc_cache_redis_ctx_t;
+
+/*
+ * free resources allocated for the per-process Redis connection context
+ */
+static apr_status_t oidc_cache_redis_free(void *ptr) {
+	oidc_cache_redis_ctx_t *rctx = (oidc_cache_redis_ctx_t *) ptr;
+	if ((rctx != NULL) && (rctx->ctx != NULL)) {
+		redisFree(rctx->ctx);
+		rctx->ctx = NULL;
+	}
+	return APR_SUCCESS;
+}
+
 /*
  * connect to Redis server
  */
-static redisContext * oidc_cache_redis_connect(request_rec *r,
+static oidc_cache_redis_ctx_t * oidc_cache_redis_connect(request_rec *r,
 		oidc_cache_cfg_redis_t *context) {
 
 	/* see if we already have a connection by looking it up in the process context */
-	redisContext *ctx = NULL;
-	apr_pool_userdata_get((void **) &ctx, OIDC_CACHE_REDIS_CONTEXT,
+	oidc_cache_redis_ctx_t *rctx = NULL;
+
+	apr_pool_userdata_get((void **) &rctx, OIDC_CACHE_REDIS_CONTEXT,
 			r->server->process->pool);
 
-	if (ctx == NULL) {
-
-		/* no connection, connect to the configured Redis server */
-		ctx = redisConnect(context->host_str, context->port);
-
-		/* check for errors */
-		if ((ctx == NULL) || (ctx->err != 0)) {
-			oidc_error(r, "failed to connect to Redis server (%s:%d): '%s'",
-					context->host_str, context->port, ctx->errstr);
-			return NULL;
-		}
+	if (rctx == NULL) {
+		rctx = apr_pcalloc(r->server->process->pool,
+				sizeof(oidc_cache_redis_ctx_t));
+		rctx->ctx = NULL;
 
 		/* store the connection in the process context */
-		apr_pool_userdata_set(ctx, OIDC_CACHE_REDIS_CONTEXT,
-				(apr_status_t (*)(void *)) redisFree, r->server->process->pool);
-
-		/* log the connection */
-		oidc_debug(r, "successfully connected to Redis server (%s:%d)",
-				context->host_str, context->port);
+		apr_pool_userdata_set(rctx, OIDC_CACHE_REDIS_CONTEXT,
+				oidc_cache_redis_free, r->server->process->pool);
 	}
 
-	return ctx;
+	if (rctx->ctx == NULL) {
+
+		/* no connection, connect to the configured Redis server */
+		rctx->ctx = redisConnect(context->host_str, context->port);
+
+		/* check for errors */
+		if ((rctx->ctx == NULL) || (rctx->ctx->err != 0)) {
+			oidc_error(r, "failed to connect to Redis server (%s:%d): '%s'",
+					context->host_str, context->port,
+					rctx->ctx != NULL ? rctx->ctx->errstr : "");
+			oidc_cache_redis_free(rctx);
+		} else {
+			/* log the connection */
+			oidc_debug(r, "successfully connected to Redis server (%s:%d)",
+					context->host_str, context->port);
+		}
+	}
+
+	return rctx;
 }
 
 /*
@@ -197,7 +225,7 @@ static redisContext * oidc_cache_redis_connect(request_rec *r,
 static redisReply* oidc_cache_redis_command(request_rec *r,
 		oidc_cache_cfg_redis_t *context, const char *format, ...) {
 
-	redisContext *ctx = NULL;
+	oidc_cache_redis_ctx_t *rctx = NULL;
 	redisReply *reply = NULL;
 	int i = 0;
 
@@ -205,35 +233,38 @@ static redisReply* oidc_cache_redis_command(request_rec *r,
 	for (i = 0; i < 2; i++) {
 
 		/* connect */
-		ctx = oidc_cache_redis_connect(r, context);
-		if (ctx == NULL)
+		rctx = oidc_cache_redis_connect(r, context);
+		if ((rctx == NULL) || (rctx->ctx == NULL))
 			break;
 
 		if (context->passwd != NULL) {
-			redisAppendCommand(ctx, apr_psprintf(r->pool, "AUTH %s", context->passwd));
+			redisAppendCommand(rctx->ctx,
+					apr_psprintf(r->pool, "AUTH %s", context->passwd));
 		}
 
 		/* execute the command */
 		va_list args;
 		va_start(args, format);
-		redisvAppendCommand(ctx, format, args);
+		redisvAppendCommand(rctx->ctx, format, args);
 		va_end(args);
 
 		if (context->passwd != NULL) {
 			/* get the reply for the AUTH command */
-			redisGetReply(ctx, (void **)&reply);
+			redisGetReply(rctx->ctx, (void **) &reply);
 			if (reply == NULL) {
-				oidc_error(r, "authentication to the Redis server (%s:%d) failed, reply == NULL",
+				oidc_error(r,
+						"authentication to the Redis server (%s:%d) failed, reply == NULL",
 						context->host_str, context->port);
 			} else if (reply->type == REDIS_REPLY_ERROR) {
-				oidc_error(r, "authentication to the Redis server (%s:%d) failed, reply.status = %s",
+				oidc_error(r,
+						"authentication to the Redis server (%s:%d) failed, reply.status = %s",
 						context->host_str, context->port, reply->str);
 			}
 		}
 
 		/* get the reply for the actual command */
 		reply = NULL;
-		redisGetReply(ctx, (void **)&reply);
+		redisGetReply(rctx->ctx, (void **) &reply);
 
 		/* errors will result in an empty reply */
 		if (reply != NULL) {
@@ -246,12 +277,12 @@ static redisReply* oidc_cache_redis_command(request_rec *r,
 		}
 
 		/* something went wrong, log it */
-		oidc_error(r, "redisvAppendCommand/redisGetReply (%d) failed, disconnecting: '%s'", i, ctx->errstr);
+		oidc_error(r,
+				"redisvAppendCommand/redisGetReply (%d) failed, disconnecting: '%s'",
+				i, rctx->ctx->errstr);
 
 		/* cleanup, we may try again (once) after reconnecting */
-		redisFree(ctx);
-		apr_pool_userdata_set(NULL, OIDC_CACHE_REDIS_CONTEXT,
-				apr_pool_cleanup_null, r->server->process->pool);
+		oidc_cache_redis_free(rctx);
 	}
 
 	return reply;
@@ -263,8 +294,6 @@ static redisReply* oidc_cache_redis_command(request_rec *r,
 static apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section,
 		const char *key, const char **value) {
 
-	oidc_debug(r, "enter, section=\"%s\", key=\"%s\"", section, key);
-
 	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
 			&auth_openidc_module);
 	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t *) cfg->cache_cfg;
@@ -315,8 +344,6 @@ static apr_byte_t oidc_cache_redis_get(request_rec *r, const char *section,
 static apr_byte_t oidc_cache_redis_set(request_rec *r, const char *section,
 		const char *key, const char *value, apr_time_t expiry) {
 
-	oidc_debug(r, "enter, section=\"%s\", key=\"%s\"", section, key);
-
 	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
 			&auth_openidc_module);
 	oidc_cache_cfg_redis_t *context = (oidc_cache_cfg_redis_t *) cfg->cache_cfg;
@@ -374,8 +401,8 @@ static int oidc_cache_redis_destroy(server_rec *s) {
 }
 
 oidc_cache_t oidc_cache_redis = {
+		"redis",
 		1,
-		oidc_cache_redis_cfg_create,
 		oidc_cache_redis_post_config,
 		oidc_cache_redis_child_init,
 		oidc_cache_redis_get,
