@@ -116,7 +116,8 @@ int oidc_cache_shm_post_config(server_rec *s) {
 	/* initialize the whole segment to '/0' */
 	int i;
 	oidc_cache_shm_entry_t *t = apr_shm_baseaddr_get(context->shm);
-	for (i = 0; i < cfg->cache_shm_size_max; i++, OIDC_CACHE_SHM_ADD_OFFSET(t, cfg->cache_shm_entry_size_max)) {
+	for (i = 0; i < cfg->cache_shm_size_max;
+			i++, OIDC_CACHE_SHM_ADD_OFFSET(t, cfg->cache_shm_entry_size_max)) {
 		t->section_key[0] = '\0';
 		t->access = 0;
 	}
@@ -124,7 +125,9 @@ int oidc_cache_shm_post_config(server_rec *s) {
 	if (oidc_cache_mutex_post_config(s, context->mutex, "shm") == FALSE)
 		return HTTP_INTERNAL_SERVER_ERROR;
 
-	oidc_sdebug(s, "initialized shared memory with a cache size (# entries) of: %d, and a max (single) entry size of: %d", cfg->cache_shm_size_max, cfg->cache_shm_entry_size_max);
+	oidc_sdebug(s,
+			"initialized shared memory with a cache size (# entries) of: %d, and a max (single) entry size of: %d",
+			cfg->cache_shm_size_max, cfg->cache_shm_entry_size_max);
 
 	return OK;
 }
@@ -155,8 +158,6 @@ static char *oidc_cache_shm_get_key(apr_pool_t *pool, const char *section,
 static apr_byte_t oidc_cache_shm_get(request_rec *r, const char *section,
 		const char *key, const char **value) {
 
-	oidc_debug(r, "enter, section=\"%s\", key=\"%s\"", section, key);
-
 	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
 			&auth_openidc_module);
 	oidc_cache_cfg_shm_t *context = (oidc_cache_cfg_shm_t *) cfg->cache_cfg;
@@ -174,10 +175,11 @@ static apr_byte_t oidc_cache_shm_get(request_rec *r, const char *section,
 	oidc_cache_shm_entry_t *t = apr_shm_baseaddr_get(context->shm);
 
 	/* loop over the block, looking for the key */
-	for (i = 0; i < cfg->cache_shm_size_max; i++, OIDC_CACHE_SHM_ADD_OFFSET(t, cfg->cache_shm_entry_size_max)) {
+	for (i = 0; i < cfg->cache_shm_size_max;
+			i++, OIDC_CACHE_SHM_ADD_OFFSET(t, cfg->cache_shm_entry_size_max)) {
 		const char *tablekey = t->section_key;
 
-		if ( (tablekey != NULL) && (apr_strnatcmp(tablekey, section_key) == 0) ) {
+		if ((tablekey != NULL) && (apr_strnatcmp(tablekey, section_key) == 0)) {
 
 			/* found a match, check if it has expired */
 			if (t->expires > apr_time_now()) {
@@ -202,7 +204,7 @@ static apr_byte_t oidc_cache_shm_get(request_rec *r, const char *section,
 	/* release the global lock */
 	oidc_cache_mutex_unlock(r, context->mutex);
 
-	return (*value == NULL) ? FALSE : TRUE;
+	return TRUE;
 }
 
 /*
@@ -211,9 +213,6 @@ static apr_byte_t oidc_cache_shm_get(request_rec *r, const char *section,
 static apr_byte_t oidc_cache_shm_set(request_rec *r, const char *section,
 		const char *key, const char *value, apr_time_t expiry) {
 
-	oidc_debug(r, "enter, section=\"%s\", key=\"%s\", value size=%llu", section,
-			key, value ? (unsigned long long )strlen(value) : 0);
-
 	oidc_cfg *cfg = ap_get_module_config(r->server->module_config,
 			&auth_openidc_module);
 	oidc_cache_cfg_shm_t *context = (oidc_cache_cfg_shm_t *) cfg->cache_cfg;
@@ -234,9 +233,15 @@ static apr_byte_t oidc_cache_shm_set(request_rec *r, const char *section,
 	}
 
 	/* check that the passed in value is valid */
-	if ((value != NULL) && (strlen(value) > (cfg->cache_shm_entry_size_max - sizeof(oidc_cache_shm_entry_t)))) {
-		oidc_error(r, "could not store value since value size is too large (%llu > %lu); consider increasing OIDCCacheShmEntrySizeMax",
-				(unsigned long long)strlen(value), (unsigned long)(cfg->cache_shm_entry_size_max - sizeof(oidc_cache_shm_entry_t)));
+	if ((value != NULL)
+			&& (strlen(value)
+					> (cfg->cache_shm_entry_size_max
+							- sizeof(oidc_cache_shm_entry_t)))) {
+		oidc_error(r,
+				"could not store value since value size is too large (%llu > %lu); consider increasing " OIDCCacheShmEntrySizeMax "",
+				(unsigned long long )strlen(value),
+				(unsigned long )(cfg->cache_shm_entry_size_max
+						- sizeof(oidc_cache_shm_entry_t)));
 		return FALSE;
 	}
 
@@ -254,7 +259,8 @@ static apr_byte_t oidc_cache_shm_set(request_rec *r, const char *section,
 	match = NULL;
 	free = NULL;
 	lru = t;
-	for (i = 0; i < cfg->cache_shm_size_max; i++, OIDC_CACHE_SHM_ADD_OFFSET(t, cfg->cache_shm_entry_size_max)) {
+	for (i = 0; i < cfg->cache_shm_size_max;
+			i++, OIDC_CACHE_SHM_ADD_OFFSET(t, cfg->cache_shm_entry_size_max)) {
 
 		/* see if this slot is free */
 		if (t->section_key[0] == '\0') {
@@ -288,7 +294,7 @@ static apr_byte_t oidc_cache_shm_set(request_rec *r, const char *section,
 		age = (current_time - lru->access) / 1000000;
 		if (age < 3600) {
 			oidc_warn(r,
-					"dropping LRU entry with age = %" APR_TIME_T_FMT "s, which is less than one hour; consider increasing the shared memory caching space (which is %d now) with the (global) OIDCCacheShmMax setting.",
+					"dropping LRU entry with age = %" APR_TIME_T_FMT "s, which is less than one hour; consider increasing the shared memory caching space (which is %d now) with the (global) " OIDCCacheShmMax " setting.",
 					age, cfg->cache_shm_size_max);
 		}
 	}
@@ -325,9 +331,13 @@ static int oidc_cache_shm_destroy(server_rec *s) {
 	apr_status_t rv = APR_SUCCESS;
 
 	if (context->shm) {
-		rv = apr_shm_destroy(context->shm);
-		oidc_sdebug(s, "apr_shm_destroy returned: %d", rv);
+		apr_global_mutex_lock(context->mutex->mutex);
+		if (*context->mutex->sema == 1) {
+			rv = apr_shm_destroy(context->shm);
+			oidc_sdebug(s, "apr_shm_destroy returned: %d", rv);
+		}
 		context->shm = NULL;
+		apr_global_mutex_unlock(context->mutex->mutex);
 	}
 
 	oidc_cache_mutex_destroy(s, context->mutex);
@@ -336,8 +346,8 @@ static int oidc_cache_shm_destroy(server_rec *s) {
 }
 
 oidc_cache_t oidc_cache_shm = {
+		"shm",
 		0,
-		oidc_cache_shm_cfg_create,
 		oidc_cache_shm_post_config,
 		oidc_cache_shm_child_init,
 		oidc_cache_shm_get,
