@@ -114,6 +114,11 @@ const char *oidc_jwt_hdr_get(oidc_jwt_t *jwt, const char *key) {
 	return hdr ? cjose_header_get(hdr, key, &cjose_err) : NULL;
 }
 
+/*
+ * {"alg":"none"}
+ */
+#define OIDC_JOSE_HDR_ALG_NONE "eyJhbGciOiJub25lIn0"
+
 /*
  * perform compact serialization on a JWT and return the resulting string
  */
@@ -121,7 +126,7 @@ char *oidc_jwt_serialize(apr_pool_t *pool, oidc_jwt_t *jwt,
 		oidc_jose_error_t *err) {
 	cjose_err cjose_err;
 	const char *cser = NULL;
-	if (strcmp(jwt->header.alg, "none") != 0) {
+	if (strcmp(jwt->header.alg, CJOSE_HDR_ALG_NONE) != 0) {
 		if (cjose_jws_export(jwt->cjose_jws, &cser, &cjose_err) == FALSE) {
 			oidc_jose_error(err, "cjose_jws_export failed: %s",
 					oidc_cjose_e2s(pool, cjose_err));
@@ -134,15 +139,15 @@ char *oidc_jwt_serialize(apr_pool_t *pool, oidc_jwt_t *jwt,
 
 		char *out = NULL;
 		size_t out_len;
-		if (cjose_base64url_encode((const uint8_t *)s_payload, strlen(s_payload), &out, &out_len,
-				&cjose_err) == FALSE)
+		if (cjose_base64url_encode((const uint8_t *) s_payload,
+				strlen(s_payload), &out, &out_len, &cjose_err) == FALSE)
 			return FALSE;
-		cser = apr_pstrndup(pool, out, out_len);
+		cser = apr_pstrmemdup(pool, out, out_len);
 		cjose_get_dealloc()(out);
 
 		free(s_payload);
 
-		cser = apr_psprintf(pool, "eyJhbGciOiJub25lIn0.%s.", cser);
+		cser = apr_psprintf(pool, "%s.%s.", OIDC_JOSE_HDR_ALG_NONE, cser);
 	}
 	return apr_pstrdup(pool, cser);
 }
@@ -183,7 +188,7 @@ int oidc_jwt_alg2kty(oidc_jwt_t *jwt) {
 /*
  * return the key size for an algorithm
  */
-int oidc_alg2keysize(const char *alg) {
+unsigned int oidc_alg2keysize(const char *alg) {
 
 	if (alg == NULL)
 		return 0;
@@ -195,14 +200,17 @@ int oidc_alg2keysize(const char *alg) {
 	if (strcmp(alg, CJOSE_HDR_ALG_A256KW) == 0)
 		return 32;
 
-	if ((strcmp(alg, "RS256") == 0) || (strcmp(alg, "PS256") == 0)
-			|| (strcmp(alg, "HS256") == 0))
+	if ((strcmp(alg, CJOSE_HDR_ALG_RS256) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_PS256) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_HS256) == 0))
 		return 32;
-	if ((strcmp(alg, "RS384") == 0) || (strcmp(alg, "PS384") == 0)
-			|| (strcmp(alg, "HS384") == 0))
+	if ((strcmp(alg, CJOSE_HDR_ALG_RS384) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_PS384) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_HS384) == 0))
 		return 48;
-	if ((strcmp(alg, "RS512") == 0) || (strcmp(alg, "PS512") == 0)
-			|| (strcmp(alg, "HS512") == 0))
+	if ((strcmp(alg, CJOSE_HDR_ALG_RS512) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_PS512) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_HS512) == 0))
 		return 64;
 
 	return 0;
@@ -219,6 +227,10 @@ static oidc_jwk_t *oidc_jwk_new(apr_pool_t *pool) {
 static apr_byte_t oidc_jwk_parse_rsa_x5c(apr_pool_t *pool, json_t *json,
 		cjose_jwk_t **jwk, oidc_jose_error_t *err);
 
+#define OIDC_JOSE_HDR_KTY      "kty"
+#define OIDC_JOSE_HDR_KTY_RSA  "RSA"
+#define OIDC_JOSE_HDR_X5C      "x5c"
+
 /*
  * parse a JSON object with an RSA "x5c" JWK representation in to a cjose JWK object
  */
@@ -236,26 +248,30 @@ static cjose_jwk_t *oidc_jwk_parse_rsa_x5c_spec(apr_pool_t *pool,
 	}
 
 	char *kty = NULL;
-	oidc_jose_get_string(pool, json, "kty", FALSE, &kty, NULL);
+	oidc_jose_get_string(pool, json, OIDC_JOSE_HDR_KTY, FALSE, &kty, NULL);
 	if (kty == NULL) {
-		oidc_jose_error(err, "no key type \"kty\" found in JWK JSON value");
+		oidc_jose_error(err,
+				"no key type \"" OIDC_JOSE_HDR_KTY "\" found in JWK JSON value");
 		goto end;
 	}
 
-	if (apr_strnatcmp(kty, "RSA") != 0) {
-		oidc_jose_error(err, "no RSA key type found JWK JSON value");
+	if (apr_strnatcmp(kty, OIDC_JOSE_HDR_KTY_RSA) != 0) {
+		oidc_jose_error(err,
+				"no \"" OIDC_JOSE_HDR_KTY_RSA "\" key type found JWK JSON value");
 		goto end;
 	}
 
-	json_t *v = json_object_get(json, "x5c");
+	json_t *v = json_object_get(json, OIDC_JOSE_HDR_X5C);
 	if (v == NULL) {
-		oidc_jose_error(err, "no x5c key found in JWK JSON value");
+		oidc_jose_error(err,
+				"no \"" OIDC_JOSE_HDR_X5C "\" key found in JWK JSON value");
 		goto end;
 	}
 
 	oidc_jwk_parse_rsa_x5c(pool, json, &cjose_jwk, err);
 
-	end: if (json)
+end:
+	if (json)
 		json_decref(json);
 
 	return cjose_jwk;
@@ -314,8 +330,7 @@ void oidc_jwk_list_destroy(apr_pool_t *pool, apr_hash_t *keys) {
 	apr_hash_index_t *hi = NULL;
 	if (keys == NULL)
 		return;
-	for (hi = apr_hash_first(pool, keys); hi; hi =
-			apr_hash_next(hi)) {
+	for (hi = apr_hash_first(pool, keys); hi; hi = apr_hash_next(hi)) {
 		oidc_jwk_t *jwk = NULL;
 		apr_hash_this(hi, NULL, NULL, (void **) &jwk);
 		oidc_jwk_destroy(jwk);
@@ -370,7 +385,7 @@ static apr_byte_t oidc_jose_hash_and_base64url_encode(apr_pool_t *pool,
 	if (cjose_base64url_encode(hashed, hashed_len, &out, &out_len,
 			&cjose_err) == FALSE)
 		return FALSE;
-	*output = apr_pstrndup(pool, out, out_len);
+	*output = apr_pstrmemdup(pool, out, out_len);
 	cjose_get_dealloc()(out);
 	return TRUE;
 }
@@ -388,8 +403,8 @@ static apr_byte_t oidc_jwk_set_or_generate_kid(apr_pool_t *pool,
 		jwk_kid = apr_pstrdup(pool, s_kid);
 	} else {
 		/* calculate a unique key identifier (kid) by fingerprinting the key params */
-		if (oidc_jose_hash_and_base64url_encode(pool, "sha256", key_params,
-				key_params_len, &jwk_kid) == FALSE) {
+		if (oidc_jose_hash_and_base64url_encode(pool, OIDC_JOSE_ALG_SHA256,
+				key_params, key_params_len, &jwk_kid) == FALSE) {
 			oidc_jose_error(err, "oidc_jose_hash_and_base64urlencode failed");
 			return FALSE;
 		}
@@ -410,7 +425,8 @@ static apr_byte_t oidc_jwk_set_or_generate_kid(apr_pool_t *pool,
  * create an "oct" symmetric JWK
  */
 oidc_jwk_t *oidc_jwk_create_symmetric_key(apr_pool_t *pool, const char *skid,
-		const unsigned char *key, unsigned int key_len, apr_byte_t set_kid, oidc_jose_error_t *err) {
+		const unsigned char *key, unsigned int key_len, apr_byte_t set_kid,
+		oidc_jose_error_t *err) {
 
 	cjose_err cjose_err;
 	cjose_jwk_t *cjose_jwk = cjose_jwk_create_oct_spec(key, key_len,
@@ -441,10 +457,11 @@ oidc_jwk_t *oidc_jwk_create_symmetric_key(apr_pool_t *pool, const char *skid,
  */
 static apr_byte_t oidc_jose_array_has_string(apr_array_header_t *haystack,
 		const char *needle) {
-	int i;
-	for (i = 0; i < haystack->nelts; i++) {
+	int i = 0;
+	while (i < haystack->nelts) {
 		if (apr_strnatcmp(((const char**) haystack->elts)[i], needle) == 0)
 			return TRUE;
+		i++;
 	}
 	return FALSE;
 }
@@ -454,21 +471,21 @@ static apr_byte_t oidc_jose_array_has_string(apr_array_header_t *haystack,
  */
 apr_array_header_t *oidc_jose_jws_supported_algorithms(apr_pool_t *pool) {
 	apr_array_header_t *result = apr_array_make(pool, 12, sizeof(const char*));
-	*(const char**) apr_array_push(result) = "RS256";
-	*(const char**) apr_array_push(result) = "RS384";
-	*(const char**) apr_array_push(result) = "RS512";
-	*(const char**) apr_array_push(result) = "PS256";
-	*(const char**) apr_array_push(result) = "PS384";
-	*(const char**) apr_array_push(result) = "PS512";
-	*(const char**) apr_array_push(result) = "HS256";
-	*(const char**) apr_array_push(result) = "HS384";
-	*(const char**) apr_array_push(result) = "HS512";
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_RS256;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_RS384;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_RS512;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_PS256;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_PS384;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_PS512;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_HS256;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_HS384;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_HS512;
 #if (OIDC_JOSE_EC_SUPPORT)
-	*(const char**) apr_array_push(result) = "ES256";
-	*(const char**) apr_array_push(result) = "ES384";
-	*(const char**) apr_array_push(result) = "ES512";
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_ES256;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_ES384;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_ES512;
 #endif
-	*(const char**) apr_array_push(result) = "none";
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_NONE;
 	return result;
 }
 
@@ -486,11 +503,11 @@ apr_byte_t oidc_jose_jws_algorithm_is_supported(apr_pool_t *pool,
  */
 apr_array_header_t *oidc_jose_jwe_supported_algorithms(apr_pool_t *pool) {
 	apr_array_header_t *result = apr_array_make(pool, 4, sizeof(const char*));
-	*(const char**) apr_array_push(result) = "RSA1_5";
-	*(const char**) apr_array_push(result) = "A128KW";
-	*(const char**) apr_array_push(result) = "A192KW";
-	*(const char**) apr_array_push(result) = "A256KW";
-	*(const char**) apr_array_push(result) = "RSA-OAEP";
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_RSA1_5;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_A128KW;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_A192KW;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_A256KW;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ALG_RSA_OAEP;
 	return result;
 }
 
@@ -508,13 +525,11 @@ apr_byte_t oidc_jose_jwe_algorithm_is_supported(apr_pool_t *pool,
  */
 apr_array_header_t *oidc_jose_jwe_supported_encryptions(apr_pool_t *pool) {
 	apr_array_header_t *result = apr_array_make(pool, 5, sizeof(const char*));
-	*(const char**) apr_array_push(result) = "A128CBC-HS256";
-	*(const char**) apr_array_push(result) = "A192CBC-HS384";
-	*(const char**) apr_array_push(result) = "A256CBC-HS512";
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ENC_A128CBC_HS256;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ENC_A192CBC_HS384;
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ENC_A256CBC_HS512;
 #if (OIDC_JOSE_GCM_SUPPORT)
-	*(const char**) apr_array_push(result) = "A128GCM";
-	*(const char**) apr_array_push(result) = "A192GCM";
-	*(const char**) apr_array_push(result) = "A256GCM";
+	*(const char**) apr_array_push(result) = CJOSE_HDR_ENC_A256GCM;
 #endif
 	return result;
 }
@@ -577,6 +592,11 @@ static apr_byte_t oidc_jose_get_timestamp(apr_pool_t *pool, json_t *json,
 	return TRUE;
 }
 
+#define OIDC_JOSE_JWT_ISS             "iss"
+#define OIDC_JOSE_JWT_SUB             "sub"
+#define OIDC_JOSE_JWT_EXP             "exp"
+#define OIDC_JOSE_JWT_IAT             "iat"
+
 /*
  * parse JWT payload
  */
@@ -602,22 +622,24 @@ static apr_byte_t oidc_jose_parse_payload(apr_pool_t *pool,
 		return FALSE;
 	}
 
-	/* get the (optional) "issuer" value from the JSON payload */
-	oidc_jose_get_string(pool, payload->value.json, "iss", FALSE, &payload->iss,
+	/* get the (optional) "iss" value from the JSON payload */
+	oidc_jose_get_string(pool, payload->value.json, OIDC_JOSE_JWT_ISS, FALSE,
+			&payload->iss,
 			NULL);
 
 	/* get the (optional) "exp" value from the JSON payload */
-	oidc_jose_get_timestamp(pool, payload->value.json, "exp", FALSE,
+	oidc_jose_get_timestamp(pool, payload->value.json, OIDC_JOSE_JWT_EXP, FALSE,
 			&payload->exp,
 			NULL);
 
 	/* get the (optional) "iat" value from the JSON payload */
-	oidc_jose_get_timestamp(pool, payload->value.json, "iat", FALSE,
+	oidc_jose_get_timestamp(pool, payload->value.json, OIDC_JOSE_JWT_IAT, FALSE,
 			&payload->iat,
 			NULL);
 
 	/* get the (optional) "sub" value from the JSON payload */
-	oidc_jose_get_string(pool, payload->value.json, "sub", FALSE, &payload->sub,
+	oidc_jose_get_string(pool, payload->value.json, OIDC_JOSE_JWT_SUB, FALSE,
+			&payload->sub,
 			NULL);
 
 	return TRUE;
@@ -679,14 +701,15 @@ static uint8_t *oidc_jwe_decrypt_impl(apr_pool_t *pool, cjose_jwe_t *jwe,
  * decrypt a JSON Web Token
  */
 apr_byte_t oidc_jwe_decrypt(apr_pool_t *pool, const char *input_json,
-		apr_hash_t *keys, char **s_json, oidc_jose_error_t *err, apr_byte_t import_must_succeed) {
+		apr_hash_t *keys, char **s_json, oidc_jose_error_t *err,
+		apr_byte_t import_must_succeed) {
 	cjose_err cjose_err;
 	cjose_jwe_t *jwe = cjose_jwe_import(input_json, strlen(input_json),
 			&cjose_err);
 	if (jwe != NULL) {
 		size_t content_len = 0;
-		uint8_t *decrypted = oidc_jwe_decrypt_impl(pool, jwe, keys, &content_len,
-				err);
+		uint8_t *decrypted = oidc_jwe_decrypt_impl(pool, jwe, keys,
+				&content_len, err);
 		if (decrypted != NULL) {
 			decrypted[content_len] = '\0';
 			*s_json = apr_pstrdup(pool, (const char *) decrypted);
@@ -715,6 +738,8 @@ apr_byte_t oidc_jwt_parse(apr_pool_t *pool, const char *input_json,
 		return FALSE;
 
 	*j_jwt = oidc_jwt_new(pool, FALSE, FALSE);
+	if (*j_jwt == NULL)
+		return FALSE;
 	oidc_jwt_t *jwt = *j_jwt;
 
 	jwt->cjose_jws = cjose_jws_import(s_json, strlen(s_json), &cjose_err);
@@ -727,7 +752,7 @@ apr_byte_t oidc_jwt_parse(apr_pool_t *pool, const char *input_json,
 	}
 
 	cjose_header_t *hdr = cjose_jws_get_protected(jwt->cjose_jws);
-	jwt->header.value.json = json_deep_copy((json_t *)hdr);
+	jwt->header.value.json = json_deep_copy((json_t *) hdr);
 	char *str = json_dumps(jwt->header.value.json,
 			JSON_PRESERVE_ORDER | JSON_COMPACT);
 	jwt->header.value.str = apr_pstrdup(pool, str);
@@ -784,7 +809,7 @@ void oidc_jwt_destroy(oidc_jwt_t *jwt) {
 apr_byte_t oidc_jwt_sign(apr_pool_t *pool, oidc_jwt_t *jwt, oidc_jwk_t *jwk,
 		oidc_jose_error_t *err) {
 
-	cjose_header_t *hdr = (cjose_header_t *)jwt->header.value.json;
+	cjose_header_t *hdr = (cjose_header_t *) jwt->header.value.json;
 
 	if (jwt->header.alg)
 		oidc_jwt_hdr_set(jwt, CJOSE_HDR_ALG, jwt->header.alg);
@@ -829,7 +854,7 @@ void EVP_MD_CTX_free(EVP_MD_CTX *ctx) {
 apr_byte_t oidc_jwt_encrypt(apr_pool_t *pool, oidc_jwt_t *jwe, oidc_jwk_t *jwk,
 		const char *payload, char **serialized, oidc_jose_error_t *err) {
 
-	cjose_header_t *hdr = (cjose_header_t *)jwe->header.value.json;
+	cjose_header_t *hdr = (cjose_header_t *) jwe->header.value.json;
 
 	if (jwe->header.alg)
 		oidc_jwt_hdr_set(jwe, CJOSE_HDR_ALG, jwe->header.alg);
@@ -861,6 +886,17 @@ apr_byte_t oidc_jwt_encrypt(apr_pool_t *pool, oidc_jwt_t *jwe, oidc_jwk_t *jwk,
 	return TRUE;
 }
 
+#define OIDC_JOSE_CJOSE_VERSION_DEPRECATED "0.4."
+
+/*
+ * check for a version of cjose < 0.5.0 that has a version of
+ * cjose_jws_verify that resources after a verification failure
+ */
+apr_byte_t oidc_jose_version_deprecated(apr_pool_t *pool) {
+	char *version = apr_pstrdup(pool, cjose_version());
+	return (strstr(version, OIDC_JOSE_CJOSE_VERSION_DEPRECATED) == version);
+}
+
 /*
  * verify the signature on a JWT
  */
@@ -880,7 +916,8 @@ apr_byte_t oidc_jwt_verify(apr_pool_t *pool, oidc_jwt_t *jwt, apr_hash_t *keys,
 			if (rc == FALSE) {
 				oidc_jose_error(err, "cjose_jws_verify failed: %s",
 						oidc_cjose_e2s(pool, cjose_err));
-				jwt->cjose_jws = NULL;
+				if (oidc_jose_version_deprecated(pool))
+					jwt->cjose_jws = NULL;
 			}
 		} else {
 			oidc_jose_error(err, "could not find key with kid: %s",
@@ -898,7 +935,8 @@ apr_byte_t oidc_jwt_verify(apr_pool_t *pool, oidc_jwt_t *jwt, apr_hash_t *keys,
 				if (rc == FALSE) {
 					oidc_jose_error(err, "cjose_jws_verify failed: %s",
 							oidc_cjose_e2s(pool, cjose_err));
-					jwt->cjose_jws = NULL;
+					if (oidc_jose_version_deprecated(pool))
+						jwt->cjose_jws = NULL;
 				}
 			}
 			if ((rc == TRUE) || (jwt->cjose_jws == NULL))
@@ -964,20 +1002,23 @@ apr_byte_t oidc_jose_hash_bytes(apr_pool_t *pool, const char *s_digest,
  * return the OpenSSL hash algorithm associated with a specified JWT algorithm
  */
 static char *oidc_jose_alg_to_openssl_digest(const char *alg) {
-	if ((strcmp(alg, "RS256") == 0) || (strcmp(alg, "PS256") == 0)
-			|| (strcmp(alg, "HS256") == 0) || (strcmp(alg, "ES256") == 0)) {
-		return "sha256";
+	if ((strcmp(alg, CJOSE_HDR_ALG_RS256) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_PS256) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_HS256) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_ES256) == 0)) {
+		return LN_sha256;
 	}
-	if ((strcmp(alg, "RS384") == 0) || (strcmp(alg, "PS384") == 0)
-			|| (strcmp(alg, "HS384") == 0) || (strcmp(alg, "ES384") == 0)) {
-		return "sha384";
+	if ((strcmp(alg, CJOSE_HDR_ALG_RS384) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_PS384) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_HS384) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_ES384) == 0)) {
+		return LN_sha384;
 	}
-	if ((strcmp(alg, "RS512") == 0) || (strcmp(alg, "PS512") == 0)
-			|| (strcmp(alg, "HS512") == 0) || (strcmp(alg, "ES512") == 0)) {
-		return "sha512";
-	}
-	if (strcmp(alg, "NONE") == 0) {
-		return "NONE";
+	if ((strcmp(alg, CJOSE_HDR_ALG_RS512) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_PS512) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_HS512) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_ES512) == 0)) {
+		return LN_sha512;
 	}
 	return NULL;
 }
@@ -1005,16 +1046,22 @@ apr_byte_t oidc_jose_hash_string(apr_pool_t *pool, const char *alg,
  * return hash length
  */
 int oidc_jose_hash_length(const char *alg) {
-	if ((strcmp(alg, "RS256") == 0) || (strcmp(alg, "PS256") == 0)
-			|| (strcmp(alg, "HS256") == 0) || (strcmp(alg, "ES256") == 0)) {
+	if ((strcmp(alg, CJOSE_HDR_ALG_RS256) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_PS256) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_HS256) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_ES256) == 0)) {
 		return 32;
 	}
-	if ((strcmp(alg, "RS384") == 0) || (strcmp(alg, "PS384") == 0)
-			|| (strcmp(alg, "HS384") == 0) || (strcmp(alg, "ES384") == 0)) {
+	if ((strcmp(alg, CJOSE_HDR_ALG_RS384) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_PS384) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_HS384) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_ES384) == 0)) {
 		return 48;
 	}
-	if ((strcmp(alg, "RS512") == 0) || (strcmp(alg, "PS512") == 0)
-			|| (strcmp(alg, "HS512") == 0) || (strcmp(alg, "ES512") == 0)) {
+	if ((strcmp(alg, CJOSE_HDR_ALG_RS512) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_PS512) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_HS512) == 0)
+			|| (strcmp(alg, CJOSE_HDR_ALG_ES512) == 0)) {
 		return 64;
 	}
 	return 0;
@@ -1109,7 +1156,6 @@ static apr_byte_t oidc_jwk_rsa_bio_to_jwk(apr_pool_t *pool, BIO *input,
 	rv = TRUE;
 
 end:
-
 	if (pkey)
 		EVP_PKEY_free(pkey);
 	if (x509)
@@ -1147,13 +1193,15 @@ static apr_byte_t oidc_jwk_parse_rsa_key(apr_pool_t *pool, int is_private_key,
 	rv = TRUE;
 
 end:
-
 	if (input)
 		BIO_free(input);
 
 	return rv;
 }
 
+#define OIDC_JOSE_CERT_BEGIN "-----BEGIN CERTIFICATE-----"
+#define OIDC_JOSE_CERT_END   "-----END CERTIFICATE-----"
+
 /*
  * parse an RSA key from a JSON object in to a cjose JWK object
  */
@@ -1163,15 +1211,16 @@ static apr_byte_t oidc_jwk_parse_rsa_x5c(apr_pool_t *pool, json_t *json,
 	apr_byte_t rv = FALSE;
 
 	/* get the "x5c" array element from the JSON object */
-	json_t *v = json_object_get(json, "x5c");
+	json_t *v = json_object_get(json, OIDC_JOSE_HDR_X5C);
 	if (v == NULL) {
-		oidc_jose_error(err, "JSON key \"%s\" could not be found", "x5c");
+		oidc_jose_error(err, "JSON key \"%s\" could not be found",
+				OIDC_JOSE_HDR_X5C);
 		return FALSE;
 	}
 	if (!json_is_array(v)) {
 		oidc_jose_error(err,
 				"JSON key \"%s\" was found but its value is not a JSON array",
-				"x5c");
+				OIDC_JOSE_HDR_X5C);
 		return FALSE;
 	}
 
@@ -1191,12 +1240,12 @@ static apr_byte_t oidc_jwk_parse_rsa_x5c(apr_pool_t *pool, json_t *json,
 	/* PEM-format it */
 	const int len = 75;
 	int i = 0;
-	char *s = apr_psprintf(pool, "-----BEGIN CERTIFICATE-----\n");
+	char *s = apr_psprintf(pool, "%s\n", OIDC_JOSE_CERT_BEGIN);
 	while (i < strlen(s_x5c)) {
-		s = apr_psprintf(pool, "%s%s\n", s, apr_pstrndup(pool, s_x5c + i, len));
+		s = apr_psprintf(pool, "%s%s\n", s, apr_pstrmemdup(pool, s_x5c + i, len));
 		i += len;
 	}
-	s = apr_psprintf(pool, "%s-----END CERTIFICATE-----\n", s);
+	s = apr_psprintf(pool, "%s%s\n", s, OIDC_JOSE_CERT_END);
 
 	BIO *input = NULL;
 
@@ -1223,9 +1272,9 @@ static apr_byte_t oidc_jwk_parse_rsa_x5c(apr_pool_t *pool, json_t *json,
 /*
  * parse an X.509 PEM formatted certificate file with an RSA public key to a JWK struct
  */
-apr_byte_t oidc_jwk_parse_rsa_private_key(apr_pool_t *pool,
+apr_byte_t oidc_jwk_parse_rsa_private_key(apr_pool_t *pool, const char *kid,
 		const char *filename, oidc_jwk_t **jwk, oidc_jose_error_t *err) {
-	return oidc_jwk_parse_rsa_key(pool, TRUE, NULL, filename, jwk, err);
+	return oidc_jwk_parse_rsa_key(pool, TRUE, kid, filename, jwk, err);
 }
 
 /*
