@@ -62,7 +62,12 @@
  * validate an access token against the validation endpoint of the Authorization server and gets a response back
  */
 static apr_byte_t oidc_oauth_validate_access_token(request_rec *r, oidc_cfg *c,
-		const char *token, const char **response) {
+		const char *token, char **response) {
+	
+	oidc_debug(r, "enter");
+
+	char *basic_auth = NULL;
+	char *bearer_auth = NULL;
 
 	/* assemble parameters to call the token endpoint for validation */
 	apr_table_t *params = apr_table_make(r->pool, 4);
@@ -74,40 +79,38 @@ static apr_byte_t oidc_oauth_validate_access_token(request_rec *r, oidc_cfg *c,
 	/* add the access_token itself */
 	apr_table_addn(params, c->oauth.introspection_token_param_name, token);
 
-	/* see if we want to do basic auth or post-param-based auth */
-	const char *basic_auth = NULL;
-	if ((c->oauth.client_id != NULL) && (c->oauth.client_secret != NULL)) {
-		if ((c->oauth.introspection_endpoint_auth != NULL)
-				&& (apr_strnatcmp(c->oauth.introspection_endpoint_auth,
-						"client_secret_post") == 0)) {
-			apr_table_addn(params, "client_id", c->oauth.client_id);
-			apr_table_addn(params, "client_secret", c->oauth.client_secret);
-		} else {
-			basic_auth = apr_psprintf(r->pool, "%s:%s", c->oauth.client_id,
-					c->oauth.client_secret);
-		}
-	}
+	/* add the token endpoint authentication credentials */
+	if (oidc_proto_token_endpoint_auth(r, c,
+			c->oauth.introspection_endpoint_auth, c->oauth.client_id,
+			c->oauth.client_secret, c->oauth.introspection_endpoint_url, params,
+			&basic_auth, &bearer_auth) == FALSE)
+		return FALSE;
 
 	/* call the endpoint with the constructed parameter set and return the resulting response */
-	return apr_strnatcmp(c->oauth.introspection_endpoint_method, OIDC_INTROSPECTION_METHOD_GET) == 0 ?
-			oidc_util_http_get(r, c->oauth.introspection_endpoint_url, params,
-					basic_auth, NULL, c->oauth.ssl_validate_server, response,
-					c->http_timeout_long, c->outgoing_proxy,
-					oidc_dir_cfg_pass_cookies(r), c->oauth.introspection_endpoint_tls_client_cert, c->oauth.introspection_endpoint_tls_client_key):
-			oidc_util_http_post_form(r, c->oauth.introspection_endpoint_url,
-					params, basic_auth, NULL, c->oauth.ssl_validate_server,
-					response, c->http_timeout_long, c->outgoing_proxy,
-					oidc_dir_cfg_pass_cookies(r), c->oauth.introspection_endpoint_tls_client_cert, c->oauth.introspection_endpoint_tls_client_key);
+	return apr_strnatcmp(c->oauth.introspection_endpoint_method,
+			OIDC_INTROSPECTION_METHOD_GET) == 0 ?
+					oidc_util_http_get(r, c->oauth.introspection_endpoint_url, params,
+							basic_auth, bearer_auth, c->oauth.ssl_validate_server, response,
+							c->http_timeout_long, c->outgoing_proxy,
+							oidc_dir_cfg_pass_cookies(r),
+							oidc_util_get_full_path(r->pool, c->oauth.introspection_endpoint_tls_client_cert),
+							oidc_util_get_full_path(r->pool, c->oauth.introspection_endpoint_tls_client_key)) :
+							oidc_util_http_post_form(r, c->oauth.introspection_endpoint_url,
+									params, basic_auth, bearer_auth, c->oauth.ssl_validate_server,
+									response, c->http_timeout_long, c->outgoing_proxy,
+									oidc_dir_cfg_pass_cookies(r),
+									oidc_util_get_full_path(r->pool, c->oauth.introspection_endpoint_tls_client_cert),
+									oidc_util_get_full_path(r->pool, c->oauth.introspection_endpoint_tls_client_key));
 }
 
 /*
  * get the authorization header that should contain a bearer token
  */
-static apr_byte_t oidc_oauth_get_bearer_token(request_rec *r,
+apr_byte_t oidc_oauth_get_bearer_token(request_rec *r,
 		const char **access_token) {
 
 	/* get the directory specific setting on how the token can be passed in */
-	int accept_token_in = oidc_cfg_dir_accept_token_in(r);
+	apr_byte_t accept_token_in = oidc_cfg_dir_accept_token_in(r);
 	const char *cookie_name = oidc_cfg_dir_accept_token_in_option(r,
 			OIDC_OAUTH_ACCEPT_TOKEN_IN_OPTION_COOKIE_NAME);
 
@@ -119,14 +122,14 @@ static apr_byte_t oidc_oauth_get_bearer_token(request_rec *r,
 			|| (accept_token_in == OIDC_OAUTH_ACCEPT_TOKEN_IN_DEFAULT)) {
 
 		/* get the authorization header */
-		const char *auth_line;
-		auth_line = apr_table_get(r->headers_in, "Authorization");
+		const char *auth_line = oidc_util_hdr_in_authorization_get(r);
 		if (auth_line) {
 			oidc_debug(r, "authorization header found");
 
 			/* look for the Bearer keyword */
-			if (apr_strnatcasecmp(ap_getword(r->pool, &auth_line, ' '),
-					"Bearer") == 0) {
+			if (apr_strnatcasecmp(
+					ap_getword(r->pool, &auth_line, OIDC_CHAR_SPACE),
+					OIDC_PROTO_BEARER) == 0) {
 
 				/* skip any spaces after the Bearer keyword */
 				while (apr_isspace(*auth_line)) {
@@ -148,7 +151,7 @@ static apr_byte_t oidc_oauth_get_bearer_token(request_rec *r,
 			&& (accept_token_in & OIDC_OAUTH_ACCEPT_TOKEN_IN_POST)) {
 		apr_table_t *params = apr_table_make(r->pool, 8);
 		if (oidc_util_read_post_params(r, params) == TRUE) {
-			*access_token = apr_table_get(params, "access_token");
+			*access_token = apr_table_get(params, OIDC_PROTO_ACCESS_TOKEN);
 		}
 	}
 
@@ -156,7 +159,7 @@ static apr_byte_t oidc_oauth_get_bearer_token(request_rec *r,
 			&& (accept_token_in & OIDC_OAUTH_ACCEPT_TOKEN_IN_QUERY)) {
 		apr_table_t *params = apr_table_make(r->pool, 8);
 		oidc_util_read_form_encoded_params(r, params, r->args);
-		*access_token = apr_table_get(params, "access_token");
+		*access_token = apr_table_get(params, OIDC_PROTO_ACCESS_TOKEN);
 	}
 
 	if ((*access_token == NULL)
@@ -265,23 +268,25 @@ static apr_byte_t oidc_oauth_parse_and_cache_token_expiry(request_rec *r,
 	return TRUE;
 }
 
+#define OIDC_OAUTH_CACHE_KEY_RESPONSE  "r"
+#define OIDC_OAUTH_CACHE_KEY_TIMESTAMP "t"
+
 static apr_byte_t oidc_oauth_cache_access_token(request_rec *r, oidc_cfg *c,
 		apr_time_t cache_until, const char *access_token, json_t *json) {
 
 	oidc_debug(r, "caching introspection result");
 
 	json_t *cache_entry = json_object();
-	json_object_set(cache_entry, "response", json);
-	json_object_set_new(cache_entry, "timestamp",
+	json_object_set(cache_entry, OIDC_OAUTH_CACHE_KEY_RESPONSE, json);
+	json_object_set_new(cache_entry, OIDC_OAUTH_CACHE_KEY_TIMESTAMP,
 			json_integer(apr_time_sec(apr_time_now())));
-	char *cache_value = json_dumps(cache_entry, 0);
+	char *cache_value = oidc_util_encode_json_object(r, cache_entry,
+			JSON_COMPACT);
 
 	/* set it in the cache so subsequent request don't need to validate the access_token and get the claims anymore */
-	c->cache->set(r, OIDC_CACHE_SECTION_ACCESS_TOKEN, access_token, cache_value,
-			cache_until);
+	oidc_cache_set_access_token(r, access_token, cache_value, cache_until);
 
 	json_decref(cache_entry);
-	free(cache_value);
 
 	return TRUE;
 }
@@ -289,26 +294,22 @@ static apr_byte_t oidc_oauth_cache_access_token(request_rec *r, oidc_cfg *c,
 static apr_byte_t oidc_oauth_get_cached_access_token(request_rec *r,
 		oidc_cfg *c, const char *access_token, json_t **json) {
 	json_t *cache_entry = NULL;
-	const char *s_cache_entry = NULL;
-	json_error_t json_error;
+	char *s_cache_entry = NULL;
 
 	/* see if we've got the claims for this access_token cached already */
-	c->cache->get(r, OIDC_CACHE_SECTION_ACCESS_TOKEN, access_token,
-			&s_cache_entry);
+	oidc_cache_get_access_token(r, access_token, &s_cache_entry);
 
 	if (s_cache_entry == NULL)
 		return FALSE;
 
 	/* json decode the cache entry */
-	cache_entry = json_loads(s_cache_entry, 0, &json_error);
-	if (cache_entry == NULL) {
-		oidc_error(r, "cached JSON was corrupted: %s", json_error.text);
+	if (oidc_util_decode_json_object(r, s_cache_entry, &cache_entry) == FALSE) {
 		*json = NULL;
 		return FALSE;
 	}
 
 	/* compare the timestamp against the freshness requirement */
-	json_t *v = json_object_get(cache_entry, "timestamp");
+	json_t *v = json_object_get(cache_entry, OIDC_OAUTH_CACHE_KEY_TIMESTAMP);
 	apr_time_t now = apr_time_sec(apr_time_now());
 	int token_introspection_interval = oidc_cfg_token_introspection_interval(r);
 	if ((token_introspection_interval > 0)
@@ -332,7 +333,8 @@ static apr_byte_t oidc_oauth_get_cached_access_token(request_rec *r,
 			s_cache_entry);
 
 	/* we've got a cached introspection result that is still valid for this path's requirements */
-	*json = json_deep_copy(json_object_get(cache_entry, "response"));
+	*json = json_deep_copy(
+			json_object_get(cache_entry, OIDC_OAUTH_CACHE_KEY_RESPONSE));
 
 	json_decref(cache_entry);
 	return TRUE;
@@ -351,7 +353,7 @@ static apr_byte_t oidc_oauth_resolve_access_token(request_rec *r, oidc_cfg *c,
 
 	if (result == NULL) {
 
-		const char *s_json = NULL;
+		char *s_json = NULL;
 
 		/* not cached, go out and validate the access_token against the Authorization server and get the JSON claims back */
 		if (oidc_oauth_validate_access_token(r, c, access_token,
@@ -365,33 +367,36 @@ static apr_byte_t oidc_oauth_resolve_access_token(request_rec *r, oidc_cfg *c,
 		if (oidc_util_decode_json_and_check_error(r, s_json, &result) == FALSE)
 			return FALSE;
 
-		json_t *active = json_object_get(result, "active");
+		json_t *active = json_object_get(result, OIDC_PROTO_ACTIVE);
 		apr_time_t cache_until;
 		if (active != NULL) {
 
 			if (json_is_boolean(active)) {
 				if (!json_is_true(active)) {
 					oidc_debug(r,
-							"\"active\" boolean object with value \"false\" found in response JSON object");
+							"\"%s\" boolean object with value \"false\" found in response JSON object",
+							OIDC_PROTO_ACTIVE);
 					json_decref(result);
 					return FALSE;
 				}
 			} else if (json_is_string(active)) {
 				if (apr_strnatcasecmp(json_string_value(active), "true") != 0) {
 					oidc_debug(r,
-							"\"active\" string object with value that is not equal to \"true\" found in response JSON object: %s",
-							json_string_value(active));
+							"\"%s\" string object with value that is not equal to \"true\" found in response JSON object: %s",
+							OIDC_PROTO_ACTIVE, json_string_value(active));
 					json_decref(result);
 					return FALSE;
 				}
 			} else {
 				oidc_debug(r,
-						"no \"active\" boolean or string object found in response JSON object");
+						"no \"%s\" boolean or string object found in response JSON object",
+						OIDC_PROTO_ACTIVE);
 				json_decref(result);
 				return FALSE;
 			}
 
-			if (oidc_oauth_parse_and_cache_token_expiry(r, c, result, "exp",
+			if (oidc_oauth_parse_and_cache_token_expiry(r, c, result,
+					OIDC_CLAIM_EXP,
 					TRUE, FALSE, &cache_until) == FALSE) {
 				json_decref(result);
 				return FALSE;
@@ -407,7 +412,7 @@ static apr_byte_t oidc_oauth_resolve_access_token(request_rec *r, oidc_cfg *c,
 					c->oauth.introspection_token_expiry_claim_name,
 					apr_strnatcmp(
 							c->oauth.introspection_token_expiry_claim_format,
-							"absolute") == 0,
+							OIDC_CLAIM_FORMAT_ABSOLUTE) == 0,
 							c->oauth.introspection_token_expiry_claim_required,
 							&cache_until) == FALSE) {
 				json_decref(result);
@@ -423,17 +428,19 @@ static apr_byte_t oidc_oauth_resolve_access_token(request_rec *r, oidc_cfg *c,
 	}
 
 	/* return the access_token JSON object */
-	json_t *tkn = json_object_get(result, "access_token");
+	json_t *tkn = json_object_get(result, OIDC_PROTO_ACCESS_TOKEN);
 	if ((tkn != NULL) && (json_is_object(tkn))) {
 
 		/*
 		 * assume PingFederate validation: copy over those claims from the access_token
 		 * that are relevant for authorization purposes
 		 */
-		json_object_set(tkn, "client_id", json_object_get(result, "client_id"));
-		json_object_set(tkn, "scope", json_object_get(result, "scope"));
+		json_object_set(tkn, OIDC_PROTO_CLIENT_ID,
+				json_object_get(result, OIDC_PROTO_CLIENT_ID));
+		json_object_set(tkn, OIDC_PROTO_SCOPE,
+				json_object_get(result, OIDC_PROTO_SCOPE));
 
-		//oidc_oauth_spaced_string_to_array(r, result, "scope", tkn, "scopes");
+		//oidc_oauth_spaced_string_to_array(r, result, OIDC_PROTO_SCOPE, tkn, "scopes");
 
 		/* return only the pimped access_token results */
 		*token = json_deep_copy(tkn);
@@ -442,54 +449,15 @@ static apr_byte_t oidc_oauth_resolve_access_token(request_rec *r, oidc_cfg *c,
 
 	} else {
 
-		//oidc_oauth_spaced_string_to_array(r, result, "scope", result, "scopes");
+		//oidc_oauth_spaced_string_to_array(r, result, OIDC_PROTO_SCOPE, result, "scopes");
 
 		/* assume spec compliant introspection */
 		*token = result;
 
 	}
 
-	char *s_token = json_dumps(*token, 0);
-	*response = apr_pstrdup(r->pool, s_token);
-	free(s_token);
-
-	return TRUE;
-}
-
-/*
- * set the unique user identifier that will be propagated in the Apache r->user and REMOTE_USER variables
- */
-static apr_byte_t oidc_oauth_set_remote_user(request_rec *r, oidc_cfg *c,
-		json_t *token) {
-
-	/* get the configured claim name to populate REMOTE_USER with (defaults to "Username") */
-	char *claim_name = apr_pstrdup(r->pool,
-			c->oauth.remote_user_claim.claim_name);
-
-	/* get the claim value from the resolved token JSON response to use as the REMOTE_USER key */
-	json_t *username = json_object_get(token, claim_name);
-	if ((username == NULL) || (!json_is_string(username))) {
-		oidc_warn(r, "response JSON object did not contain a \"%s\" string",
-				claim_name);
-		return FALSE;
-	}
-
-	r->user = apr_pstrdup(r->pool, json_string_value(username));
-
-	if (c->oauth.remote_user_claim.reg_exp != NULL) {
-
-		char *error_str = NULL;
-		if (oidc_util_regexp_first_match(r->pool, r->user,
-				c->oauth.remote_user_claim.reg_exp, &r->user,
-				&error_str) == FALSE) {
-			oidc_error(r, "oidc_util_regexp_first_match failed: %s", error_str);
-			r->user = NULL;
-			return FALSE;
-		}
-	}
-
-	oidc_debug(r, "set REMOTE_USER to claim %s=%s", claim_name,
-			json_string_value(username));
+	/* stringify the response */
+	*response = oidc_util_encode_json_object(r, *token, JSON_COMPACT);
 
 	return TRUE;
 }
@@ -500,7 +468,6 @@ static apr_byte_t oidc_oauth_set_remote_user(request_rec *r, oidc_cfg *c,
  * TODO: document that we're reusing the following settings from the OIDC config section:
  *       - JWKs URI refresh interval
  *       - encryption key material (OIDCPrivateKeyFiles)
- *       - iat slack (OIDCIDTokenIatSlack)
  *
  * OIDCOAuthRemoteUserClaim client_id
  * # 32x 61 hex
@@ -531,9 +498,11 @@ static apr_byte_t oidc_oauth_validate_jwt_access_token(request_rec *r,
 	oidc_debug(r, "successfully parsed JWT with header: %s",
 			jwt->header.value.str);
 
-	/* validate the access token JWT, validating optional exp + iat */
-	if (oidc_proto_validate_jwt(r, jwt, NULL, FALSE, FALSE,
-			c->provider.idtoken_iat_slack) == FALSE) {
+	/*
+	 * validate the access token JWT by validating the (optional) exp claim
+	 * don't enforce anything around iat since it doesn't make much sense for access tokens
+	 */
+	if (oidc_proto_validate_jwt(r, jwt, NULL, FALSE, FALSE, -1) == FALSE) {
 		oidc_jwt_destroy(jwt);
 		return FALSE;
 	}
@@ -571,19 +540,45 @@ static apr_byte_t oidc_oauth_validate_jwt_access_token(request_rec *r,
  */
 int oidc_oauth_return_www_authenticate(request_rec *r, const char *error,
 		const char *error_description) {
-	char *hdr = apr_psprintf(r->pool, "Bearer");
+	char *hdr = apr_psprintf(r->pool, "%s", OIDC_PROTO_BEARER);
 	if (ap_auth_name(r) != NULL)
-		hdr = apr_psprintf(r->pool, "%s realm=\"%s\"", hdr, ap_auth_name(r));
+		hdr = apr_psprintf(r->pool, "%s %s=\"%s\"", hdr, OIDC_PROTO_REALM,
+				ap_auth_name(r));
 	if (error != NULL)
-		hdr = apr_psprintf(r->pool, "%s%s error=\"%s\"", hdr,
-				(ap_auth_name(r) ? "," : ""), error);
+		hdr = apr_psprintf(r->pool, "%s%s %s=\"%s\"", hdr,
+				(ap_auth_name(r) ? "," : ""), OIDC_PROTO_ERROR, error);
 	if (error_description != NULL)
-		hdr = apr_psprintf(r->pool, "%s, error_description=\"%s\"", hdr,
-				error_description);
-	apr_table_setn(r->err_headers_out, "WWW-Authenticate", hdr);
+		hdr = apr_psprintf(r->pool, "%s, %s=\"%s\"", hdr,
+				OIDC_PROTO_ERROR_DESCRIPTION, error_description);
+	oidc_util_hdr_err_out_add(r, OIDC_HTTP_HDR_WWW_AUTHENTICATE, hdr);
 	return HTTP_UNAUTHORIZED;
 }
 
+/*
+ * set the unique user identifier that will be propagated in the Apache r->user and REMOTE_USER variables
+ */
+static apr_byte_t oidc_oauth_set_request_user(request_rec *r, oidc_cfg *c,
+		json_t *token) {
+	char *remote_user = NULL;
+
+	if (oidc_get_remote_user(r, c->oauth.remote_user_claim.claim_name,
+			c->oauth.remote_user_claim.reg_exp, c->oauth.remote_user_claim.replace, token, &remote_user) == FALSE) {
+		oidc_error(r,
+				"" OIDCOAuthRemoteUserClaim " is set to \"%s\", but could not set the remote user based the available claims for the user",
+				c->oauth.remote_user_claim.claim_name);
+		return FALSE;
+	}
+
+	r->user = remote_user;
+    oidc_debug(r, "set user to \"%s\" based on claim: \"%s\"%s", r->user,
+               c->oauth.remote_user_claim.claim_name,
+               c->oauth.remote_user_claim.reg_exp ?
+               apr_psprintf(r->pool, " and expression: \"%s\" and replace string: \"%s\"",
+                            c->oauth.remote_user_claim.reg_exp, c->oauth.remote_user_claim.replace) :
+               "");
+	return TRUE;
+}
+
 /*
  * main routine: handle OAuth 2.0 authentication/authorization
  */
@@ -604,14 +599,17 @@ int oidc_oauth_check_userid(request_rec *r, oidc_cfg *c) {
 					"recycling user '%s' from initial request for sub-request",
 					r->user);
 
+			/* strip any cookies that we need to */
+			oidc_strip_cookies(r);
+
 			return OK;
 		}
 
 		/* check if this is a request for the public (encryption) keys */
-	} else if ((c->redirect_uri != NULL)
-			&& (oidc_util_request_matches_url(r, c->redirect_uri))) {
+	} else if (oidc_util_request_matches_url(r, oidc_get_redirect_uri(r, c))) {
 
-		if (oidc_util_request_has_parameter(r, "jwks")) {
+		if (oidc_util_request_has_parameter(r,
+				OIDC_REDIRECT_URI_REQUEST_JWKS)) {
 
 			return oidc_handle_jwks(r, c);
 
@@ -623,9 +621,14 @@ int oidc_oauth_check_userid(request_rec *r, oidc_cfg *c) {
 
 	/* get the bearer access token from the Authorization header */
 	const char *access_token = NULL;
-	if (oidc_oauth_get_bearer_token(r, &access_token) == FALSE)
-		return oidc_oauth_return_www_authenticate(r, "invalid_request",
-				"No bearer token found in the request");
+	if (oidc_oauth_get_bearer_token(r, &access_token) == FALSE) {
+		if (r->method_number == M_OPTIONS) {
+			r->user = "";
+			return OK;
+		}
+		return oidc_oauth_return_www_authenticate(r,
+				OIDC_PROTO_ERR_INVALID_REQUEST, "No bearer token found in the request");
+	}
 
 	/* validate the obtained access token against the OAuth AS validation endpoint */
 	json_t *token = NULL;
@@ -636,32 +639,36 @@ int oidc_oauth_check_userid(request_rec *r, oidc_cfg *c) {
 		/* we'll validate the token remotely */
 		if (oidc_oauth_resolve_access_token(r, c, access_token, &token,
 				&s_token) == FALSE)
-			return oidc_oauth_return_www_authenticate(r, "invalid_token",
+			return oidc_oauth_return_www_authenticate(r,
+					OIDC_PROTO_ERR_INVALID_TOKEN,
 					"Reference token could not be introspected");
 	} else {
 		/* no introspection endpoint is set, assume the token is a JWT and validate it locally */
 		if (oidc_oauth_validate_jwt_access_token(r, c, access_token, &token,
 				&s_token) == FALSE)
-			return oidc_oauth_return_www_authenticate(r, "invalid_token",
-					"JWT token could not be validated");
+			return oidc_oauth_return_www_authenticate(r,
+					OIDC_PROTO_ERR_INVALID_TOKEN, "JWT token could not be validated");
 	}
 
 	/* check that we've got something back */
 	if (token == NULL) {
 		oidc_error(r, "could not resolve claims (token == NULL)");
-		return oidc_oauth_return_www_authenticate(r, "invalid_token",
+		return oidc_oauth_return_www_authenticate(r,
+				OIDC_PROTO_ERR_INVALID_TOKEN,
 				"No claims could be parsed from the token");
 	}
 
 	/* store the parsed token (cq. the claims from the response) in the request state so it can be accessed by the authz routines */
-	oidc_request_state_set(r, OIDC_CLAIMS_SESSION_KEY, (const char *) s_token);
+	oidc_request_state_set(r, OIDC_REQUEST_STATE_KEY_CLAIMS,
+			(const char *) s_token);
 
-	/* set the REMOTE_USER variable */
-	if (oidc_oauth_set_remote_user(r, c, token) == FALSE) {
+	/* set the request user */
+	if (oidc_oauth_set_request_user(r, c, token) == FALSE) {
+		json_decref(token);
 		oidc_error(r,
 				"remote user could not be set, aborting with HTTP_UNAUTHORIZED");
-		return oidc_oauth_return_www_authenticate(r, "invalid_token",
-				"Could not set remote user");
+		return oidc_oauth_return_www_authenticate(r,
+				OIDC_PROTO_ERR_INVALID_TOKEN, "Could not set remote user");
 	}
 
 	/*
@@ -672,27 +679,27 @@ int oidc_oauth_check_userid(request_rec *r, oidc_cfg *c) {
 
 	/* set the user authentication HTTP header if set and required */
 	char *authn_header = oidc_cfg_dir_authn_header(r);
-	int pass_headers = oidc_cfg_dir_pass_info_in_headers(r);
-	int pass_envvars = oidc_cfg_dir_pass_info_in_envvars(r);
+	apr_byte_t pass_headers = oidc_cfg_dir_pass_info_in_headers(r);
+	apr_byte_t pass_envvars = oidc_cfg_dir_pass_info_in_envvars(r);
 
-	if ((r->user != NULL) && (authn_header != NULL)) {
-		oidc_debug(r, "setting authn header (%s) to: %s", authn_header,
-				r->user);
-		apr_table_set(r->headers_in, authn_header, r->user);
-	}
+	if ((r->user != NULL) && (authn_header != NULL))
+		oidc_util_hdr_in_set(r, authn_header, r->user);
 
 	/* set the resolved claims in the HTTP headers for the target application */
-	oidc_util_set_app_infos(r, token, c->claim_prefix, c->claim_delimiter,
-			pass_headers, pass_envvars);
+	oidc_util_set_app_infos(r, token, oidc_cfg_claim_prefix(r),
+			c->claim_delimiter, pass_headers, pass_envvars);
 
 	/* set the access_token in the app headers */
 	if (access_token != NULL) {
-		oidc_util_set_app_info(r, "access_token", access_token,
+		oidc_util_set_app_info(r, OIDC_APP_INFO_ACCESS_TOKEN, access_token,
 				OIDC_DEFAULT_HEADER_PREFIX, pass_headers, pass_envvars);
 	}
 
 	/* free JSON resources */
 	json_decref(token);
 
+	/* strip any cookies that we need to */
+	oidc_strip_cookies(r);
+
 	return OK;
 }
