@@ -308,7 +308,8 @@ const char *oidc_parse_cache_type(apr_pool_t *pool, const char *arg,
  */
 const char *oidc_parse_session_type(apr_pool_t *pool, const char *arg,
 		int *type, int *persistent) {
-	static char *options[] = {
+	static char *options[] =
+	{
 			OIDC_SESSION_TYPE_SERVER_CACHE_STR,
 			OIDC_SESSION_TYPE_SERVER_CACHE_STR OIDC_SESSION_TYPE_SEPARATOR OIDC_SESSION_TYPE_PERSISTENT,
 			OIDC_SESSION_TYPE_CLIENT_COOKIE_STR,
@@ -377,15 +378,18 @@ const char *oidc_parse_boolean(apr_pool_t *pool, const char *arg,
 #define OIDC_ENDPOINT_AUTH_CLIENT_SECRET_BASIC "client_secret_basic"
 #define OIDC_ENDPOINT_AUTH_CLIENT_SECRET_JWT   "client_secret_jwt"
 #define OIDC_ENDPOINT_AUTH_PRIVATE_KEY_JWT     "private_key_jwt"
+#define OIDC_ENDPOINT_AUTH_NONE                "none"
 
 /*
  * check if the provided endpoint authentication method is supported
  */
-static const char *oidc_valid_endpoint_auth_method_impl(apr_pool_t *pool, const char *arg, apr_byte_t has_private_key) {
+static const char *oidc_valid_endpoint_auth_method_impl(apr_pool_t *pool,
+		const char *arg, apr_byte_t has_private_key) {
 	static char *options[] = {
 			OIDC_ENDPOINT_AUTH_CLIENT_SECRET_POST,
 			OIDC_ENDPOINT_AUTH_CLIENT_SECRET_BASIC,
 			OIDC_ENDPOINT_AUTH_CLIENT_SECRET_JWT,
+			OIDC_ENDPOINT_AUTH_NONE,
 			NULL,
 			NULL };
 	if (has_private_key)
@@ -398,7 +402,8 @@ const char *oidc_valid_endpoint_auth_method(apr_pool_t *pool, const char *arg) {
 	return oidc_valid_endpoint_auth_method_impl(pool, arg, TRUE);
 }
 
-const char *oidc_valid_endpoint_auth_method_no_private_key(apr_pool_t *pool, const char *arg) {
+const char *oidc_valid_endpoint_auth_method_no_private_key(apr_pool_t *pool,
+		const char *arg) {
 	return oidc_valid_endpoint_auth_method_impl(pool, arg, FALSE);
 }
 
@@ -409,14 +414,12 @@ const char *oidc_valid_response_type(apr_pool_t *pool, const char *arg) {
 	if (oidc_proto_flow_is_supported(pool, arg) == FALSE) {
 		return apr_psprintf(pool,
 				"oidc_valid_response_type: type must be one of %s",
-				apr_array_pstrcat(pool, oidc_proto_supported_flows(pool), '|'));
+				apr_array_pstrcat(pool, oidc_proto_supported_flows(pool),
+						OIDC_CHAR_PIPE));
 	}
 	return NULL;
 }
 
-#define OIDC_PKCE_METHOD_PLAIN "plain"
-#define OIDC_PKCE_METHOD_S256  "S256"
-
 /*
  * check if the provided PKCE method is supported
  */
@@ -424,6 +427,7 @@ const char *oidc_valid_pkce_method(apr_pool_t *pool, const char *arg) {
 	static char *options[] = {
 			OIDC_PKCE_METHOD_PLAIN,
 			OIDC_PKCE_METHOD_S256,
+			OIDC_PKCE_METHOD_REFERRED_TB,
 			NULL };
 	return oidc_valid_string_option(pool, arg, options);
 }
@@ -453,7 +457,8 @@ const char *oidc_valid_signed_response_alg(apr_pool_t *pool, const char *arg) {
 				"unsupported/invalid signing algorithm '%s'; must be one of [%s]",
 				arg,
 				apr_array_pstrcat(pool,
-						oidc_jose_jws_supported_algorithms(pool), '|'));
+						oidc_jose_jws_supported_algorithms(pool),
+						OIDC_CHAR_PIPE));
 	}
 	return NULL;
 }
@@ -467,7 +472,8 @@ const char *oidc_valid_encrypted_response_alg(apr_pool_t *pool, const char *arg)
 				"unsupported/invalid encryption algorithm '%s'; must be one of [%s]",
 				arg,
 				apr_array_pstrcat(pool,
-						oidc_jose_jwe_supported_algorithms(pool), '|'));
+						oidc_jose_jwe_supported_algorithms(pool),
+						OIDC_CHAR_PIPE));
 	}
 	return NULL;
 }
@@ -481,13 +487,14 @@ const char *oidc_valid_encrypted_response_enc(apr_pool_t *pool, const char *arg)
 				"unsupported/invalid encryption type '%s'; must be one of [%s]",
 				arg,
 				apr_array_pstrcat(pool,
-						oidc_jose_jwe_supported_encryptions(pool), '|'));
+						oidc_jose_jwe_supported_encryptions(pool),
+						OIDC_CHAR_PIPE));
 	}
 	return NULL;
 }
 
 #define OIDC_SESSION_INACTIVITY_TIMEOUT_MIN 10
-#define OIDC_SESSION_INACTIVITY_TIMEOUT_MAX 86400
+#define OIDC_SESSION_INACTIVITY_TIMEOUT_MAX 3600 * 24 * 365
 
 /*
  * parse a session inactivity timeout value from the provided string
@@ -696,6 +703,58 @@ const char *oidc_parse_pass_idtoken_as(apr_pool_t *pool, const char *v1,
 	return NULL;
 }
 
+#define OIDC_PASS_USERINFO_AS_CLAIMS_STR      "claims"
+#define OIDC_PASS_USERINFO_AS_JSON_OBJECT_STR "json"
+#define OIDC_PASS_USERINFO_AS_JWT_STR         "jwt"
+
+/*
+ * convert a "pass userinfo as" value to an integer
+ */
+static int oidc_parse_pass_userinfo_as_str2int(const char *v) {
+	if (apr_strnatcmp(v, OIDC_PASS_USERINFO_AS_CLAIMS_STR) == 0)
+		return OIDC_PASS_USERINFO_AS_CLAIMS;
+	if (apr_strnatcmp(v, OIDC_PASS_USERINFO_AS_JSON_OBJECT_STR) == 0)
+		return OIDC_PASS_USERINFO_AS_JSON_OBJECT;
+	if (apr_strnatcmp(v, OIDC_PASS_USERINFO_AS_JWT_STR) == 0)
+		return OIDC_PASS_USERINFO_AS_JWT;
+	return -1;
+}
+
+/*
+ * parse a "pass id token as" value from the provided strings
+ */
+const char *oidc_parse_pass_userinfo_as(apr_pool_t *pool, const char *v1,
+		const char *v2, const char *v3, int *int_value) {
+	static char *options[] = {
+			OIDC_PASS_USERINFO_AS_CLAIMS_STR,
+			OIDC_PASS_USERINFO_AS_JSON_OBJECT_STR,
+			OIDC_PASS_USERINFO_AS_JWT_STR,
+			NULL };
+	const char *rv = NULL;
+	rv = oidc_valid_string_option(pool, v1, options);
+	if (rv != NULL)
+		return rv;
+	*int_value = oidc_parse_pass_userinfo_as_str2int(v1);
+
+	if (v2 == NULL)
+		return NULL;
+
+	rv = oidc_valid_string_option(pool, v2, options);
+	if (rv != NULL)
+		return rv;
+	*int_value |= oidc_parse_pass_userinfo_as_str2int(v2);
+
+	if (v3 == NULL)
+		return NULL;
+
+	rv = oidc_valid_string_option(pool, v3, options);
+	if (rv != NULL)
+		return rv;
+	*int_value |= oidc_parse_pass_userinfo_as_str2int(v3);
+
+	return NULL;
+}
+
 #define OIDC_OAUTH_ACCEPT_TOKEN_IN_HEADER_STR "header"
 #define OIDC_OAUTH_ACCEPT_TOKEN_IN_POST_STR   "post"
 #define OIDC_OAUTH_ACCEPT_TOKEN_IN_QUERY_STR  "query"
@@ -889,6 +948,34 @@ const char *oidc_parse_unauth_action(apr_pool_t *pool, const char *arg,
 	return NULL;
 }
 
+#define OIDC_UNAUTZ_ACTION_AUTH_STR "auth"
+#define OIDC_UNAUTZ_ACTION_401_STR  "401"
+#define OIDC_UNAUTZ_ACTION_403_STR  "403"
+
+/*
+ * parse an "unauthorized action" value from the provided string
+ */
+const char *oidc_parse_unautz_action(apr_pool_t *pool, const char *arg,
+		int *action) {
+	static char *options[] = {
+			OIDC_UNAUTZ_ACTION_AUTH_STR,
+			OIDC_UNAUTZ_ACTION_401_STR,
+			OIDC_UNAUTZ_ACTION_403_STR,
+			NULL };
+	const char *rv = oidc_valid_string_option(pool, arg, options);
+	if (rv != NULL)
+		return rv;
+
+	if (apr_strnatcmp(arg, OIDC_UNAUTZ_ACTION_AUTH_STR) == 0)
+		*action = OIDC_UNAUTZ_AUTHENTICATE;
+	else if (apr_strnatcmp(arg, OIDC_UNAUTZ_ACTION_401_STR) == 0)
+		*action = OIDC_UNAUTZ_RETURN401;
+	else if (apr_strnatcmp(arg, OIDC_UNAUTZ_ACTION_403_STR) == 0)
+		*action = OIDC_UNAUTZ_RETURN403;
+
+	return NULL;
+}
+
 /*
  * check if there's one valid entry in a string of arrays
  */
@@ -1014,3 +1101,109 @@ const char *oidc_parse_userinfo_token_method(apr_pool_t *pool, const char *arg,
 
 	return NULL;
 }
+
+/*
+ * parse an "info hook data" value from the provided string
+ */
+const char *oidc_parse_info_hook_data(apr_pool_t *pool, const char *arg,
+		apr_hash_t **hook_data) {
+	static char *options[] = {
+			OIDC_HOOK_INFO_TIMESTAMP,
+			OIDC_HOOK_INFO_ACCES_TOKEN,
+			OIDC_HOOK_INFO_ACCES_TOKEN_EXP,
+			OIDC_HOOK_INFO_ID_TOKEN,
+			OIDC_HOOK_INFO_USER_INFO,
+			OIDC_HOOK_INFO_REFRESH_TOKEN,
+			OIDC_HOOK_INFO_SESSION,
+			NULL };
+	const char *rv = oidc_valid_string_option(pool, arg, options);
+	if (rv != NULL)
+		return rv;
+	if (*hook_data == NULL)
+		*hook_data = apr_hash_make(pool);
+	apr_hash_set(*hook_data, arg, APR_HASH_KEY_STRING, arg);
+
+	return NULL;
+}
+
+#define OIDC_TOKEN_BINDING_POLICY_DISABLED_STR "disabled"
+#define OIDC_TOKEN_BINDING_POLICY_OPTIONAL_STR "optional"
+#define OIDC_TOKEN_BINDING_POLICY_REQUIRED_STR "required"
+#define OIDC_TOKEN_BINDING_POLICY_ENFORCED_STR "enforced"
+
+const char *oidc_token_binding_policy2str(apr_pool_t *pool, int v) {
+	if (v == OIDC_TOKEN_BINDING_POLICY_DISABLED)
+		return OIDC_TOKEN_BINDING_POLICY_DISABLED;
+	if (v == OIDC_TOKEN_BINDING_POLICY_OPTIONAL)
+		return OIDC_TOKEN_BINDING_POLICY_OPTIONAL_STR;
+	if (v == OIDC_TOKEN_BINDING_POLICY_REQUIRED)
+		return OIDC_TOKEN_BINDING_POLICY_REQUIRED_STR;
+	if (v == OIDC_TOKEN_BINDING_POLICY_ENFORCED)
+		return OIDC_TOKEN_BINDING_POLICY_ENFORCED_STR;
+	return NULL;
+}
+
+/*
+ * check token binding policy string value
+ */
+const char *oidc_valid_token_binding_policy(apr_pool_t *pool, const char *arg) {
+	static char *options[] = {
+			OIDC_TOKEN_BINDING_POLICY_DISABLED_STR,
+			OIDC_TOKEN_BINDING_POLICY_OPTIONAL_STR,
+			OIDC_TOKEN_BINDING_POLICY_REQUIRED_STR,
+			OIDC_TOKEN_BINDING_POLICY_ENFORCED_STR,
+			NULL };
+	return oidc_valid_string_option(pool, arg, options);
+}
+
+/*
+ * parse token binding policy
+ */
+const char *oidc_parse_token_binding_policy(apr_pool_t *pool, const char *arg,
+		int *policy) {
+	const char *rv = oidc_valid_token_binding_policy(pool, arg);
+	if (rv != NULL)
+		return rv;
+
+	if (apr_strnatcmp(arg, OIDC_TOKEN_BINDING_POLICY_DISABLED_STR) == 0)
+		*policy = OIDC_TOKEN_BINDING_POLICY_DISABLED;
+	else if (apr_strnatcmp(arg, OIDC_TOKEN_BINDING_POLICY_OPTIONAL_STR) == 0)
+		*policy = OIDC_TOKEN_BINDING_POLICY_OPTIONAL;
+	else if (apr_strnatcmp(arg, OIDC_TOKEN_BINDING_POLICY_REQUIRED_STR) == 0)
+		*policy = OIDC_TOKEN_BINDING_POLICY_REQUIRED;
+	else if (apr_strnatcmp(arg, OIDC_TOKEN_BINDING_POLICY_ENFORCED_STR) == 0)
+		*policy = OIDC_TOKEN_BINDING_POLICY_ENFORCED;
+
+	return NULL;
+}
+
+#define OIDC_AUTH_REQUEST_METHOD_GET_STR  "GET"
+#define OIDC_AUTH_REQEUST_METHOD_POST_STR "POST"
+
+/*
+ * parse method for sending the authentication request
+ */
+const char *oidc_valid_auth_request_method(apr_pool_t *pool, const char *arg) {
+	static char *options[] = {
+			OIDC_AUTH_REQUEST_METHOD_GET_STR,
+			OIDC_AUTH_REQEUST_METHOD_POST_STR,
+			NULL };
+	return oidc_valid_string_option(pool, arg, options);
+}
+
+/*
+ * parse method for sending the authentication request
+ */
+const char *oidc_parse_auth_request_method(apr_pool_t *pool, const char *arg,
+		int *method) {
+	const char *rv = oidc_valid_auth_request_method(pool, arg);
+	if (rv != NULL)
+		return rv;
+
+	if (apr_strnatcmp(arg, OIDC_AUTH_REQUEST_METHOD_GET_STR) == 0)
+		*method = OIDC_AUTH_REQUEST_METHOD_GET;
+	else if (apr_strnatcmp(arg, OIDC_AUTH_REQEUST_METHOD_POST_STR) == 0)
+		*method = OIDC_AUTH_REQUEST_METHOD_POST;
+
+	return NULL;
+}
