@@ -73,6 +73,11 @@ AP_DECLARE(void) ap_hook_child_init(
 		const char * const *aszPre, const char * const *aszSucc, int nOrder) {
 }
 
+AP_DECLARE(void) ap_hook_handler(
+	int (*handler)(request_rec *r),
+	const char * const *aszPre, const char * const *aszSucc, int nOrder) {
+}
+
 AP_DECLARE(int) ap_is_initial_req(request_rec *r) {
 	return 0;
 }
@@ -171,3 +176,7 @@ AP_DECLARE(apr_status_t) unixd_set_global_mutex_perms(
 AP_DECLARE(const char *) ap_get_server_name(request_rec *r) {
 	return "www.example.com";
 }
+
+AP_DECLARE(char *) ap_server_root_relative(apr_pool_t *p, const char *file) {
+	return "";
+}
