@@ -158,7 +158,7 @@ static char *test_jwt_parse(apr_pool_t *pool) {
 	oidc_jwt_destroy(jwt);
 	oidc_jwk_destroy(jwk);
 
-	s[5] = '.';
+	s[5] = OIDC_CHAR_DOT;
 	TST_ASSERT_ERR("corrupted header (1) oidc_jwt_parse",
 			oidc_jwt_parse(pool, s, &jwt, NULL, &err) == FALSE, pool, err);
 
@@ -867,7 +867,7 @@ static char *test_jwt_decrypt(apr_pool_t *pool) {
 			pool, err);
     oidc_jwk_destroy(jwk_s);
 
-	TST_ASSERT_STR("header.alg", jwt->header.alg, "RS256");
+	TST_ASSERT_STR("header.alg", jwt->header.alg, CJOSE_HDR_ALG_RS256);
 	TST_ASSERT_STR("payload.iss", jwt->payload.iss, "joe");
 	TST_ASSERT_LONG("payload.exp", (long )jwt->payload.exp, 1300819380L);
 
@@ -1026,19 +1026,20 @@ static char * test_proto_authorization_request(request_rec *r) {
 	provider.response_type = "code";
 	provider.auth_request_params = NULL;
 	provider.request_object = NULL;
+	provider.token_binding_policy = OIDC_TOKEN_BINDING_POLICY_OPTIONAL;
 	const char *redirect_uri = "https://www.example.com/protected/";
 	const char *state = "12345";
 
-	json_t * proto_state = json_object();
-	json_object_set_new(proto_state, "nonce", json_string("anonce"));
-	json_object_set_new(proto_state, "original_url", json_string("https://localhost/protected/index.php"));
-	json_object_set_new(proto_state, "original_method", json_string("get"));
-	json_object_set_new(proto_state, "issuer", json_string(provider.issuer));
-	json_object_set_new(proto_state, "response_type", json_string(provider.response_type));
-	json_object_set_new(proto_state, "timestamp", json_integer(apr_time_sec(apr_time_now())));
+	oidc_proto_state_t *proto_state = oidc_proto_state_new();
+	oidc_proto_state_set_nonce(proto_state, "anonce");
+	oidc_proto_state_set_original_url(proto_state, "https://localhost/protected/index.php");
+	oidc_proto_state_set_original_method(proto_state, OIDC_METHOD_GET);
+	oidc_proto_state_set_issuer(proto_state, provider.issuer);
+	oidc_proto_state_set_response_type(proto_state, provider.response_type);
+	oidc_proto_state_set_timestamp_now(proto_state);
 
 	TST_ASSERT("oidc_proto_authorization_request (1)",
-			oidc_proto_authorization_request(r, &provider, NULL, redirect_uri, state, proto_state, NULL, NULL, NULL) == HTTP_MOVED_TEMPORARILY);
+			oidc_proto_authorization_request(r, &provider, NULL, redirect_uri, state, proto_state, NULL, NULL, NULL, NULL) == HTTP_MOVED_TEMPORARILY);
 
 	TST_ASSERT_STR("oidc_proto_authorization_request (2)",
 			apr_table_get(r->headers_out, "Location"),
@@ -1188,6 +1189,15 @@ static char * test_current_url(request_rec *r) {
 	url = oidc_get_current_url(r);
 	TST_ASSERT_STR("test_headers (5)", url, "http://www.outer.com:321");
 
+	apr_table_set(r->headers_in, "X-Forwarded-Proto", "https , http");
+	url = oidc_get_current_url(r);
+	TST_ASSERT_STR("test_headers (6)", url, "https://www.outer.com:321");
+
+	apr_table_unset(r->headers_in, "X-Forwarded-Host");
+	apr_table_unset(r->headers_in, "X-Forwarded-Port");
+	url = oidc_get_current_url(r);
+	TST_ASSERT_STR("test_headers (7)", url, "https://www.example.com");
+
 	return 0;
 }
 
@@ -1268,6 +1278,7 @@ static request_rec * test_setup(apr_pool_t *pool) {
 			"https://idp.example.com/authorize";
 	cfg->provider.scope = "openid";
 	cfg->provider.client_id = "client_id";
+	cfg->provider.token_binding_policy = OIDC_TOKEN_BINDING_POLICY_OPTIONAL;
 	cfg->redirect_uri = "https://www.example.com/protected/";
 
 	oidc_dir_cfg *d_cfg = oidc_create_dir_config(request->pool, NULL);
@@ -1280,10 +1291,12 @@ static request_rec * test_setup(apr_pool_t *pool) {
 			cfg);
 	ap_set_module_config(request->per_dir_config, &auth_openidc_module, d_cfg);
 
+	cfg->crypto_passphrase = "12345678901234567890123456789012";
 	cfg->cache = &oidc_cache_shm;
 	cfg->cache_cfg = NULL;
 	cfg->cache_shm_size_max = 500;
 	cfg->cache_shm_entry_size_max = 16384 + 255 + 17;
+	cfg->cache_encrypt = 1;
 	if (cfg->cache->post_config(request->server) != OK) {
 		printf("cfg->cache->post_config failed!\n");
 		exit(-1);
